@@ -82,7 +82,9 @@
 #'   Defaults to 0.5.
 #' @param max.iter Maximum number of iterations to try to resolve overlaps.
 #'   Defaults to 10000.
-#' @param max.overlaps Exclude text labels that overlap too many things.
+#' @param max.overlaps Exclude text labels when they overlap too many other
+#'   things. For each text label, we count how many other text labels or other
+#'   data points it overlaps, and exclude the text label if it has too many overlaps.
 #'   Defaults to 10.
 #' @param direction "both", "x", or "y" -- direction in which to adjust position of labels
 #' @param seed Random seed passed to \code{\link[base]{set.seed}}. Defaults to
@@ -179,7 +181,7 @@ geom_text_repel <- function(
   show.legend = NA,
   direction = c("both","y","x"),
   seed = NA,
-  verbose = FALSE,
+  verbose = getOption("verbose", default = FALSE),
   inherit.aes = TRUE
 ) {
   if (!missing(nudge_x) || !missing(nudge_y)) {
@@ -188,6 +190,12 @@ geom_text_repel <- function(
     }
     position <- position_nudge_repel(nudge_x, nudge_y)
   }
+  # Warn about limitations of the algorithm
+  if (verbose && any(abs(data$angle %% 90) > 5)) {
+    message(
+      "ggrepel: Repulsion works correctly only for rotation angles multiple of 90 degrees"
+    )
+  }
   layer(
     data = data,
     mapping = mapping,
@@ -238,6 +246,7 @@ GeomTextRepel <- ggproto("GeomTextRepel", Geom,
     segment.curvature = 0, segment.angle = 90, segment.ncp = 1,
     segment.shape = 0.5, segment.square = TRUE, segment.squareShape = 1,
     segment.inflect = FALSE, segment.debug = FALSE,
+    arrow.fill = NULL,
     bg.colour = NA, bg.r = 0.1
   ),
 
@@ -260,13 +269,13 @@ GeomTextRepel <- ggproto("GeomTextRepel", Geom,
     ylim = c(NA, NA),
     direction = "both",
     seed = NA,
-    verbose = FALSE
+    verbose = getOption("verbose", default = FALSE)
   ) {
-    lab <- data$label
+
     if (parse) {
-      lab <- parse_safe(as.character(lab))
+      data$label <- parse_safe(as.character(data$label))
     }
-    if (!length(which(not_empty(lab)))) {
+    if (!length(data$label) || !length(which(not_empty(data$label)))) {
       return()
     }
 
@@ -275,11 +284,24 @@ GeomTextRepel <- ggproto("GeomTextRepel", Geom,
       this_orig <- sprintf("%s_orig", this_dim)
       this_nudge <- sprintf("nudge_%s", this_dim)
       if (!this_nudge %in% colnames(data)) {
+        # No nudge column exists - use current position as nudge target
         data[[this_nudge]] <- data[[this_dim]]
         if (this_orig %in% colnames(data)) {
+          # Restore original position from _orig column (e.g., from position_nudge_repel)
           data[[this_dim]] <- data[[this_orig]]
           data[[this_orig]] <- NULL
         }
+      } else if (this_orig %in% colnames(data)) {
+        # nudge column exists AND _orig exists (from position_nudge_repel)
+        # Set nudge to the current (nudged) position, restore original
+        data[[this_nudge]] <- data[[this_dim]]
+        data[[this_dim]] <- data[[this_orig]]
+        data[[this_orig]] <- NULL
+      } else if (all(data[[this_nudge]] == 0)) {
+        # nudge column exists but is all zeros and no _orig column
+        # This happens with ggplot2's position_nudge() which sets nudge_x/y = 0
+        # In this case, use current position as nudge target (no nudge offset)
+        data[[this_nudge]] <- data[[this_dim]]
       }
     }
 
@@ -295,6 +317,10 @@ GeomTextRepel <- ggproto("GeomTextRepel", Geom,
     data$nudge_y <- nudges$y - data$y
 
     # Transform limits to panel scales.
+    # Store original NA status before transformation, because coord$transform
+    # may convert NA to a valid value (e.g., when NA is a factor level).
+    xlim_na <- is.na(xlim)
+    ylim_na <- is.na(ylim)
     limits <- data.frame(x = xlim, y = ylim)
     limits <- coord$transform(limits, panel_scales)
 
@@ -306,14 +332,9 @@ GeomTextRepel <- ggproto("GeomTextRepel", Geom,
       limits$y[is.infinite(ylim)] <- ylim[is.infinite(ylim)]
     }
 
-    # Fill NAs with defaults.
-    limits$x[is.na(limits$x)] <- c(0, 1)[is.na(limits$x)]
-    limits$y[is.na(limits$y)] <- c(0, 1)[is.na(limits$y)]
-
-    # Warn about limitations of the algorithm
-    if (any(abs(data$angle) %% 90 > 5)) {
-      warn("ggrepel: Repulsion works correctly only for rotation angles multiple of 90 degrees")
-    }
+    # Fill NAs with defaults (use original NA status, not post-transform).
+    limits$x[xlim_na] <- c(0, 1)[xlim_na]
+    limits$y[ylim_na] <- c(0, 1)[ylim_na]
 
     # Convert hjust and vjust to numeric if character
     if (is.character(data$vjust)) {
@@ -326,7 +347,7 @@ GeomTextRepel <- ggproto("GeomTextRepel", Geom,
     ggname("geom_text_repel", gTree(
       limits = limits,
       data = data,
-      lab = lab,
+      lab = data$label,
       box.padding = to_unit(box.padding),
       point.padding = to_unit(point.padding),
       min.segment.length = to_unit(min.segment.length),
@@ -397,8 +418,8 @@ makeContent.textrepeltree <- function(x) {
   })
 
   # Make the repulsion reproducible if desired.
-  if (is.null(x$seed) || !is.na(x$seed)) {
-      set.seed(x$seed)
+  if (!is.null(x$seed) && is.na(x$seed)) {
+    x$seed <- sample.int(.Machine$integer.max, 1L)
   }
 
   # The points are represented by circles.
@@ -420,7 +441,7 @@ makeContent.textrepeltree <- function(x) {
   ) / 13
 
   # Repel overlapping bounding boxes away from each other.
-  repel <- repel_boxes2(
+  repel <- with_seed_null(x$seed, repel_boxes2(
     data_points     = as.matrix(x$data[,c("x","y")]),
     point_size      = point_size,
     point_padding_x = point_padding,
@@ -437,12 +458,12 @@ makeContent.textrepeltree <- function(x) {
     max_overlaps    = x$max.overlaps,
     direction       = x$direction,
     verbose         = x$verbose
-  )
+  ))
 
-  if (any(repel$too_many_overlaps)) {
-    warn(
+  if (x$verbose && any(repel$too_many_overlaps)) {
+    message(
       sprintf(
-        "ggrepel: %s unlabeled data points (too many overlaps). Consider increasing max.overlaps",
+        "ggrepel: %s unlabeled data points (too many overlaps). Consider increasing 'max.overlaps'",
         sum(repel$too_many_overlaps)
       )
     )
@@ -454,6 +475,11 @@ makeContent.textrepeltree <- function(x) {
     return(setChildren(x, grobs))
   }
 
+  width  <- convertWidth(unit(1, "npc"), "cm", valueOnly = TRUE)
+  height <- convertHeight(unit(1, "npc"), "cm", valueOnly = TRUE)
+  point_size <- x$data$point.size * .pt / .stroke / 20
+  point_padding <- convertWidth(to_unit(x$point.padding), "cm", TRUE)
+
   grobs <- lapply(seq_along(valid_strings), function(i) {
     if (!repel$too_many_overlaps[i]) {
       row <- x$data[i, , drop = FALSE]
@@ -469,7 +495,7 @@ makeContent.textrepeltree <- function(x) {
         rot = row$angle,
         box.padding = x$box.padding,
         point.size = point_size[i],
-        point.padding = x$point.padding,
+        point.padding = point_padding,
         segment.curvature = row$segment.curvature,
         segment.angle     = row$segment.angle,
         segment.ncp       = row$segment.ncp,
@@ -488,14 +514,16 @@ makeContent.textrepeltree <- function(x) {
         segment.gp = gpar(
           col = scales::alpha(row$segment.colour %||% row$colour, row$segment.alpha %||% row$alpha),
           lwd = row$segment.size * .pt,
-          lty = row$segment.linetype %||% 1
+          lty = row$segment.linetype %||% 1,
+          fill = scales::alpha(row$arrow.fill %||% row$segment.colour %||% row$colour, row$segment.alpha %||% row$alpha)
         ),
         arrow = x$arrow,
         min.segment.length = x$min.segment.length,
         hjust = row$hjust,
         vjust = row$vjust,
         bg.colour = alpha(row$bg.colour, row$alpha),
-        bg.r = row$bg.r
+        bg.r = row$bg.r,
+        dim = c(width, height)
       )
     }
   })
@@ -541,7 +569,8 @@ makeTextRepelGrobs <- function(
   hjust = 0.5,
   vjust = 0.5,
   bg.colour = NA,
-  bg.r = .1
+  bg.r = .1,
+  dim = c(5, 5)
 ) {
   stopifnot(length(label) == 1)
 
@@ -609,10 +638,10 @@ makeTextRepelGrobs <- function(
     point_inside_text <- TRUE
   }
 
-  # This seems just fine.
-  point.padding <- convertWidth(to_unit(point.padding), "native", TRUE) / 2
-
-  point_int <- intersect_line_circle(int, point_pos, (point.size + point.padding))
+  # We multiply with `dim` here to compute this in centimetres, in which
+  # the point size/padding are given.
+  point_int <- intersect_line_circle(int * dim, point_pos * dim, (point.size + point.padding))
+  point_int <- point_int / dim
 
   # Compute the distance between the data point and the edge of the text box.
   dx <- abs(int[1] - point_int[1])
@@ -630,11 +659,11 @@ makeTextRepelGrobs <- function(
     !point_inside_text &&
     d > 0 &&
     # Distance from label to point edge is greater than minimum.
-    euclid(int, point_int) > min.segment.length &&
+    (!is.na(min.segment.length) && euclid(int, point_int) > min.segment.length) &&
     # Distance from label to point edge is less than from label to point center.
     euclid(int, point_int) < euclid(int, point_pos) &&
     # Distance from label to point center is greater than point size.
-    euclid(int, point_pos) > point.size &&
+    euclid(int * dim, point_pos * dim) > point.size &&
     # Distance from label to point center is greater than from point edge to point center.
     euclid(int, point_pos) > euclid(point_int, point_pos)
   ) {
